/// <reference types="node" />
import { EventEmitter } from 'node:events';
import { type Pool as GenericPool } from 'generic-pool';
import type Postgres from 'postgres';
declare type PgPool = {
    database?: string;
    host?: string;
    idleTimeoutMillis?: number;
    max?: number;
    min?: number;
    password?: string;
    port?: number;
    ssl?: boolean;
    user?: string;
};
declare type Command = 'DELETE' | 'INSERT' | 'SELECT' | 'UPDATE';
declare type Row = any;
declare type QueryResult = {
    command: Command;
    fields: Array<{
        dataTypeID: number;
        name: string;
    }>;
    rowCount: number;
    rows: Row[];
};
declare class NotAPromise {
    private then;
    private catch;
    private finally;
}
declare type Parameter<T = SerializableParameter> = NotAPromise & {
    /**
     * Raw value to serialize
     */
    raw: T | null;
    /**
     * PostgreSQL OID of the type
     */
    type: number;
    /**
     * Serialized value
     */
    value: string | null;
};
declare type Serializable = Date | Uint8Array | boolean | never | number | string | null;
declare type ArrayParameter<T extends readonly any[] = readonly any[]> = Parameter<T | T[]> & {
    array: true;
};
declare type SerializableParameter<T = never> = ArrayParameter | Parameter<any> | ReadonlyArray<SerializableParameter<T>> | Serializable | T | never;
export declare type BridgetClient = {
    end: () => Promise<void>;
    off: (eventName: string, listener: (...args: any[]) => void) => void;
    on: (eventName: string, listener: (...args: any[]) => void) => void;
    query: (sql: string, parameters?: SerializableParameter[]) => Promise<QueryResult>;
    release: () => Promise<void>;
};
export declare const createPostgresBridge: (postgres: typeof Postgres) => {
    new (poolConfiguration: PgPool): {
        readonly poolEvents: EventEmitter;
        readonly pool: GenericPool<BridgetClient>;
        connect(): Promise<BridgetClient>;
        _pulseQueue(): void;
        _remove(client: BridgetClient): Promise<void>;
        readonly _clients: BridgetClient[];
        readonly idleCount: number;
        off(eventName: string, listener: (...args: any[]) => void): EventEmitter;
        on(eventName: string, listener: (...args: any[]) => void): EventEmitter;
        readonly totalCount: number;
        readonly waitingCount: number;
        end(): Promise<void>;
    };
};
export {};
