"use strict";
/* eslint-disable @typescript-eslint/no-explicit-any */
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.createPostgresBridge = void 0;
const node_events_1 = require("node:events");
const generic_pool_1 = __importDefault(require("generic-pool"));
const createPostgresBridge = (postgres) => {
    return class PostgresBridge {
        constructor(poolConfiguration) {
            var _a, _b;
            this.poolEvents = new node_events_1.EventEmitter();
            this.pool = generic_pool_1.default.createPool({
                create: async () => {
                    var _a, _b;
                    const connectionEvents = new node_events_1.EventEmitter();
                    const connection = postgres({
                        database: poolConfiguration.database,
                        host: (_a = poolConfiguration.host) !== null && _a !== void 0 ? _a : 'localhost',
                        idle_timeout: poolConfiguration.idleTimeoutMillis ? poolConfiguration.idleTimeoutMillis / 1000 : 0,
                        max: 1,
                        onnotice: (notice) => {
                            connectionEvents.emit('notice', {
                                code: notice.code,
                                file: notice.file,
                                line: notice.line,
                                message: notice.message,
                                routine: notice.routine,
                                severity: notice.severity,
                                where: notice.where,
                            });
                        },
                        password: poolConfiguration.password,
                        port: (_b = poolConfiguration.port) !== null && _b !== void 0 ? _b : 5432,
                        ssl: poolConfiguration.ssl,
                        username: poolConfiguration.user,
                    });
                    const compatibleConnection = {
                        end: async () => {
                            await this.pool.destroy(compatibleConnection);
                        },
                        off: connectionEvents.off.bind(connectionEvents),
                        on: connectionEvents.on.bind(connectionEvents),
                        query: async (sql, parameters) => {
                            var _a, _b;
                            // https://github.com/porsager/postgres#result-array
                            const resultArray = await connection.unsafe(sql, parameters);
                            return {
                                command: resultArray.command,
                                fields: (_b = (_a = resultArray.columns) === null || _a === void 0 ? void 0 : _a.map((column) => {
                                    return {
                                        dataTypeID: column.type,
                                        name: column.name,
                                    };
                                })) !== null && _b !== void 0 ? _b : [],
                                rowCount: resultArray.count,
                                rows: Array.from(resultArray),
                            };
                        },
                        release: async () => {
                            await this.pool.release(compatibleConnection);
                        },
                    };
                    return compatibleConnection;
                },
                destroy: async (client) => {
                    await client.end();
                },
            }, {
                max: (_a = poolConfiguration.max) !== null && _a !== void 0 ? _a : 10,
                min: (_b = poolConfiguration.min) !== null && _b !== void 0 ? _b : 0,
            });
        }
        async connect() {
            const connection = await this.pool.acquire();
            this.poolEvents.emit('connect', connection);
            return connection;
        }
        _pulseQueue() {
            // TODO implement logic equivalent to https://github.com/brianc/node-postgres/blob/master/packages/pg-pool/index.js#L109-L152
        }
        async _remove(client) {
            await this.pool.destroy(client);
            this.poolEvents.emit('remove', client);
        }
        get _clients() {
            // @ts-expect-error accessing private method
            return Array.from(this.pool._allObjects, (member) => {
                return member.obj;
            });
        }
        get idleCount() {
            return this.pool.available;
        }
        off(eventName, listener) {
            return this.poolEvents.off(eventName, listener);
        }
        on(eventName, listener) {
            return this.poolEvents.on(eventName, listener);
        }
        get totalCount() {
            return this.pool.size;
        }
        get waitingCount() {
            return this.pool.pending;
        }
        async end() {
            await this.pool.clear();
        }
    };
};
exports.createPostgresBridge = createPostgresBridge;
//# sourceMappingURL=bridge.js.map